/*
  tmask.c
  structures for trim mask

  (c) J.J.Green 2002
  $Id: tmask.c,v 1.3 2008/09/26 20:40:09 jjg Exp $
*/

#include <string.h>
#include <stdlib.h>

#include "tmask.h"

#define RANGE_ALL "."

static trange_t parse_channel(const char*,int);
static int valid_channel(trange_t);
static int channel_max(int);

static char* default_tstring[4] = {"255",
				   RANGE_ALL "/0",
				   "255/255/255",
				   RANGE_ALL "/" RANGE_ALL "/" RANGE_ALL "/0"};

extern int tmask_valid(tmask_t* tmask)
{
  return tmask && tmask->model != MODEL_INVALID;
}

extern int tmask_channels(tmask_t* tmask)
{
  /* see tmask.h */

  return tmask->model;
}

extern const char* tmask_string_default(int channels)
{
  return default_tstring[channels-1];
}

extern tmask_t tmask_parse(const char* s,int bitdepth)
{
  int      n;
  trange_t channel[4];
  tmask_t  tmask = {0};
  char    *dup,*tok;

  /* until this is reset, tmask.model indicates a parse error */

  tmask.model = MODEL_INVALID;

  if (s == NULL) return tmask;

  if ((dup = strdup(s)) == NULL) return tmask;

  /* inspect each substring and count them */

  n = 0;

  for (tok = strtok(dup,"/") ; (tok != NULL) && (n<4) ; tok = strtok(NULL,"/"))
    if (valid_channel(channel[n] = parse_channel(tok,bitdepth))) n++;

  free(dup);

  /* there's a flash way to do this, but this is clearer */

  switch (n)
    {
    case 1:
      tmask.model  = MODEL_GREY;
      tmask.channel.grey = channel[0];
      break;
    case 2:
      tmask.model   = MODEL_GREYA;
      tmask.channel.grey  = channel[0];
      tmask.channel.alpha = channel[1];
      break;
    case 3:
      tmask.model   = MODEL_RGB;
      tmask.channel.red   = channel[0];
      tmask.channel.green = channel[1];
      tmask.channel.blue  = channel[2];
      break;
    case 4:
      tmask.model   = MODEL_RGBA;
      tmask.channel.red   = channel[0];
      tmask.channel.green = channel[1];
      tmask.channel.blue  = channel[2];
      tmask.channel.alpha = channel[3];
      break;
    default:
      return tmask;
    }

  return tmask;
}

static int valid_channel(trange_t range)
{
  return (range.min > range.max ? 0 : 1);
}

static int channel_max(int bitdepth)
{
  return 255;
}
      
static trange_t parse_channel(const char* s,int bitdepth)
{
  trange_t range;
  char* hyph;

  /* initialise to invalid range */

  range.min = 1;
  range.max = 0;

  if (strcmp(s,RANGE_ALL) == 0)
    {
      range.min = 0;
      range.max = channel_max(bitdepth);
    }
  else
    {
      if ((hyph=strchr(s,'-')) != NULL)
	{
	  range.min = atoi(s);
	  range.max = atoi(hyph+1);
	}
      else
	range.min = range.max = atoi(s);
    }      
  
  return range;
}

static int channel_match(trange_t,png_byte);

extern int tmask_match(tmask_t tmask,const png_bytep s)
{
  int n;

  n = tmask_channels(&tmask);

  switch (n)
    {
    case MODEL_GREY:
      if (!channel_match(tmask.channel.grey,s[0])) return 0;
      break;
    case MODEL_GREYA:
      if (
	  !channel_match(tmask.channel.grey,s[0]) ||
	  !channel_match(tmask.channel.alpha,s[1])
	  ) return 0;
      break;
    case MODEL_RGB:      
      if (
	  !channel_match(tmask.channel.red,s[0]) ||
	  !channel_match(tmask.channel.green,s[1]) ||
	  !channel_match(tmask.channel.blue,s[2])
	  ) return 0;
      break;
    case MODEL_RGBA:
      if (
	  !channel_match(tmask.channel.red,s[0]) ||
	  !channel_match(tmask.channel.green,s[1]) ||
	  !channel_match(tmask.channel.blue,s[2]) ||
	  !channel_match(tmask.channel.alpha,s[3])
	  ) return 0;
      break;
    default:
      return 0;
    }

  return 1;
}

static int channel_match(trange_t trange,png_byte c)
{
  return (((int)c < trange.min) || ((int)c > trange.max) ? 0 : 1);
}
